#include <QFileDialog>
#include <QTextStream>

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <qexserialport/qextserialenumerator.h>

#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>


#define CMD_VOLUME_UP           0
#define CMD_VOLUME_DOWN         1
#define CMD_NEXT_SONG           2
#define CMD_PREVIOUS_SONG       3
#define CMD_PAUSE               4
#define CMD_PLAY                5
#define CMD_INFO                6
#define CMD_NOCMD               7

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    cnt = 0;
    cnt2 = 0;

    //info o portech
    slotRefresh();

    connect(ui->pushButtonConnect, SIGNAL(clicked()), this, SLOT(slotNastavVysilac()));
    connect(ui->pushButtonDisconnect, SIGNAL(clicked()), this, SLOT(slotDisconnect()));
    connect(ui->pushButtonRefresh, SIGNAL(clicked()), this, SLOT(slotRefresh()));
}


MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::slotNastavVysilac(){

    vysilac = new QextSerialPort(ui->comboBoxVysilac->currentText(), QextSerialPort::EventDriven);
    vysilac->setBaudRate(BAUD115200);
    vysilac->setFlowControl(FLOW_OFF);//FLOW_OFF
    vysilac->setParity(PAR_NONE);
    vysilac->setDataBits(DATA_8);
    vysilac->setStopBits(STOP_1);

    int state = vysilac->open(QIODevice::ReadWrite);
    qDebug("vysilac->open() = %d", state);
    if(!vysilac->isOpen()) qDebug("nejde nastavit vysilac");;

    connect(vysilac, SIGNAL(readyRead()), this, SLOT(slotSignalizaceDat()));
    connect(vysilac, SIGNAL(readChannelFinished()), this, SLOT(slotKonec()));

    ui->pushButtonConnect->setEnabled(false);
    ui->pushButtonRefresh->setEnabled(false);
    ui->pushButtonDisconnect->setEnabled(true);

}


void MainWindow::prijmyOdpoved(){

  QByteArray ba = vysilac->readAll();
  cnt2 = cnt2 + ba.length();

  qDebug("cnt2: %d", cnt2);

}

void MainWindow::slotSignalizaceDat(){

    QByteArray bytes;
    int a = vysilac->bytesAvailable();


    if(a > 0){
      bytes.resize(a);
      vysilac->read(bytes.data(), bytes.size());
      ui->statusBar->showMessage(bytes.data(), 2000);
    }


    int cmd = 0;

    if(QString::compare(bytes.data(), "VOLUME_DOWN", Qt::CaseSensitive) == 0) cmd = CMD_VOLUME_DOWN;
    else if(QString::compare(bytes.data(), "VOLUME_UP", Qt::CaseSensitive) == 0) cmd = CMD_VOLUME_UP;
    else if(QString::compare(bytes.data(), "NEXT_SONG", Qt::CaseSensitive) == 0) cmd = CMD_NEXT_SONG;
    else if(QString::compare(bytes.data(), "PREVIOUS_SONG", Qt::CaseSensitive) == 0) cmd = CMD_PREVIOUS_SONG;
    else if(QString::compare(bytes.data(), "PAUSE", Qt::CaseSensitive) == 0) cmd = CMD_PAUSE;
    else if(QString::compare(bytes.data(), "PLAY", Qt::CaseSensitive) == 0) cmd = CMD_PLAY;
    else if(QString::compare(bytes.data(), "INFO", Qt::CaseSensitive) == 0) cmd = CMD_INFO;
    else cmd = CMD_NOCMD;

    char cCmd[] = "mocp";
    char argum[5];

    //argum = "-f";
    switch(cmd){

        case CMD_NEXT_SONG:
                                  strcpy(argum, "-f");
                                  break;

        case CMD_PREVIOUS_SONG:
                                  strcpy(argum, "-r");
                                  break;
        case CMD_VOLUME_DOWN:
                                  strcpy(argum, "-v-2");
                                  break;
        case CMD_VOLUME_UP:
                                  strcpy(argum, "-v+2");
                                  break;
        case CMD_PLAY:
                                  strcpy(argum, "-U");
                                  break;
        case CMD_PAUSE:
                                  strcpy(argum, "-P");
                                  break;
        case CMD_INFO:
                                  strcpy(argum, "-i");
                                  break;

        default:
                                break;
    }

    provedPrikaz(cmd, cCmd, argum);

}

void MainWindow::provedPrikaz(int cmd, char * cCmd, char * argum){

    int soubor;

    if(cmd == CMD_INFO) soubor = open("vystup.txt",   O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);

    int status;
    int pid = fork();
    if (pid == -1)
    {  qDebug("Fork selhal !\n");
       exit(-1);
    }
    if (pid == 0)
    {
        // potomek
        if(cmd == CMD_INFO){
            //přesměrování do souboru
            //dup2(soubor,1);

            // Redirect standard output
            if (soubor >= 0) {
             //close(STDOUT_FILENO);
             dup2(soubor, STDOUT_FILENO);
            }
        }

        int ret = execlp(cCmd, cCmd, argum, NULL);
        qDebug("ret = %d\n", ret);

    }
    else
    {  // rodic
       qDebug("Rodic: Cekam na syna\n");
       int spid = wait(&status);
       if (spid != pid) qDebug("Rodic: To neni muj syn !");

       qDebug("Rodic: Uz koncim");
    }


    if(cmd == CMD_INFO) {
        QFile fileQt("vystup.txt");
        if (!fileQt.open(QIODevice::ReadOnly | QIODevice::Text))
        {
            ;
        }

        QByteArray artist, song, album;
        int radek = 1;
        while (!fileQt.atEnd()) {
            QByteArray line = fileQt.readLine();

            if(radek == 4) {
                artist.append(line);
                artist = artist.mid(8, 21);
            }
            if(radek == 5){
                song.append(line);
                song = song.mid(11, 21);
            }
            if(radek == 6){
                album.append(line);
                album = album.mid(7, 21);
            }

            radek++;
        }

        fileQt.close();

        QByteArray msg;
        msg.append(song);
        msg.append(0xF3);
        msg.append(artist);
        msg.append(0xF6);
        msg.append(album);
        msg.append(0xF9);

        vysilac->write(msg.data(), msg.length());

    }
}

void MainWindow::slotKonec()
{
  qDebug("slotKonec");
}


void MainWindow::slotDisconnect(){

  vysilac->close();

  ui->pushButtonConnect->setEnabled(true);
  ui->pushButtonRefresh->setEnabled(true);
  ui->pushButtonDisconnect->setEnabled(false);


}

void MainWindow::slotRecieveMsg(QString s){
  //ui->textEdit->textCursor().insertText(s);
}

void MainWindow::slotRefresh(){

  ui->comboBoxVysilac->clear();

  QList<QextPortInfo> ports = QextSerialEnumerator::getPorts();
  for (int i = 0; i < ports.size(); i++) {
      ui->comboBoxVysilac->addItem(ports.at(i).physName);

  }
  ui->statusBar->showMessage("Refresh serial ports", 2000);
}
